// @input Component.ScriptComponent lensController
// @input SceneObject cameraPivot
// @input float distFromPlayer
// @input float lerpSpeed

var debug = true;
var isHolding = false;

function init(){
    script.createEvent("TouchStartEvent").bind(onTouchStart);
    script.createEvent("TouchEndEvent").bind(onTouchEnd);
    script.createEvent("UpdateEvent").bind(onUpdate);
    
    // Wait for .01 sec before executing a function
    var delayedEvent = script.createEvent("DelayedCallbackEvent");
    delayedEvent.bind(function(eventData)
    {
        orientObject();
    });
    
    // Start with a .01 sec delay
    delayedEvent.reset(.01);
}

function onTouchStart(event){
    isHolding = true;
    orientObject();
}

function onTouchEnd(event){
    isHolding = false;
}

function onUpdate(event) {
    if(isHolding){
        orientObject();
    }
    
    // DEBUGGING
    if(debug){
        // Get our position in world space
        var pos = script.getTransform().getWorldPosition();
        
        // Get our distance from the camera
        var dist = pos.distance(script.cameraPivot.getTransform().getWorldPosition());
        // Parse dist float to 2 decimal places
        dist = parseFloat(dist.toFixed(2));

        // Get pos local to the camera pivot, or in camera space
        var localPos = script.cameraPivot.getTransform().getInvertedWorldTransform().multiplyPoint(pos);
        localPos = new vec3(parseFloat(localPos.x.toFixed(2)), parseFloat(localPos.y.toFixed(2)), parseFloat(localPos.z.toFixed(2)));
        
        // Get our forward direction in world space
        var fwd = script.getTransform().forward;
        
        // Get our forward local to the camera pivot, or in camera space
        var localFwd = script.cameraPivot.getTransform().getInvertedWorldTransform().multiplyDirection(fwd);
        localFwd = new vec3(parseFloat(localFwd.x.toFixed(2)), parseFloat(localFwd.y.toFixed(2)), parseFloat(localFwd.z.toFixed(2)));
        
        // JS automatically calls "toString" on whatever is being added to a string
        var posStr = "localPos: " + localPos; 
        var fwdStr = "localFwd: " + localFwd;
        var distStr = "dist: " + dist; // You could also calc dist with localPos.length() 
        
        // Print vals on 3 lines
        var multiLineDebug = posStr + "\n" + fwdStr + "\n" + distStr;
        
        // PRINT TO CONSOLE
        print(multiLineDebug);
        
        // PRINT TO UI
        script.lensController.api.uiController.api.setTitle("Sphere Local to Cam");
        script.lensController.api.uiController.api.setInstructions(multiLineDebug);
    } 
}

function orientObject(){
    var dir = script.cameraPivot.getTransform().forward;
    dir = dir.normalize();
    
    var newPos = script.cameraPivot.getTransform().getWorldPosition();
    newPos = newPos.add(dir.uniformScale(script.distFromPlayer));
    
    // Lerp to this new position
    newPos = vec3.lerp(script.getTransform().getWorldPosition(), newPos, script.lerpSpeed * getDeltaTime());
    
    // Move below text
    newPos.y -= 1.5;
    script.getTransform().setWorldPosition(newPos);
    
    // Face the camera
    var newRot = quat.lookAt(dir.uniformScale(-1), vec3.up());
    script.getTransform().setWorldRotation(newRot);
}

// Expose function to this script's api 
// so it's accessible to other scripts
script.api.init = init;